#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#
# A library for access to geocode for address
#
# Copyright (C) 2011 Lorenzo Carbonell
# lorenzo.carbonell.cerezo@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#
import sys
import json
from comun import read_from_url
from sun import Sun
import locale
import datetime
import pytz
import urllib
locale.setlocale(locale.LC_MESSAGES, '')
LANG = locale.getlocale(locale.LC_MESSAGES)[0]
'''
import re
import locale
'''
'''
def urlEncodeNonAscii(b):
	language, output_encoding = locale.getdefaultlocale()
	return b.encode(output_encoding or 'utf-8')
'''

URLDIR_YAHOO = 'http://query.yahooapis.com/v1/public/yql?q=select * from geo.placefinder where text="%s" and gflags="R" and locale="%s"&format=json'
URLINV_YAHOO = 'http://query.yahooapis.com/v1/public/yql?q=select * from geo.placefinder where text="%s,%s" and gflags="R" and locale="%s"&format=json'
def s2f(cadena):
	try:
		value = float(cadena)
	except:
		value = 0.0
	return value

def fromjson2direction(json_string):
	direction = get_default_values()
	if 'city' in json_string.keys() and json_string['city']:
		direction['city'] = json_string['city']
	if 'state' in json_string.keys() and json_string['state']:
		direction['state'] = json_string['state']
	if 'country' in json_string.keys() and json_string['country']:
		direction['country'] = json_string['country']
	if 'latitude' in json_string.keys() and json_string['latitude']:
		direction['lat'] = s2f(json_string['latitude'])
	if 'longitude' in json_string.keys() and json_string['longitude']:
		direction['lng'] = s2f(json_string['longitude'])
	if 'woeid' in json_string.keys() and json_string['woeid']:
		direction['woeid'] = json_string['woeid']
	direction['search_string'] = ''
	if len(direction['city']):
		direction['search_string'] += direction['city']+','
	if len(direction['state']):
		direction['search_string'] += direction['state']+','
	if len(direction['country']):
		direction['search_string'] += direction['country']
	if direction['search_string'].endswith(','):
		direction['search_string'] = direction['search_string'][:-1]
	if s2f(direction['woeid']) == 0.0:
		direction =  get_direction(direction['search_string'])				
	return direction
	

def get_default_values():
	direction ={}
	direction['city'] = ''
	direction['state'] = ''
	direction['country'] = ''
	direction['lat'] = 0.0
	direction['lng'] = 0.0
	direction['woeid'] = ''
	direction['search_string'] = ''
	return direction
	
def is_direction_in_directions(direction,directions):
	for adirection in directions:
		if direction['search_string'] == adirection['search_string']:
			return True
	return False

def get_direction(search_string):
	directions = get_directions(search_string)
	if len(directions)>0:
		return directions[0]
	return None

def get_timezoneId(lat,lon):
	print('****** Requesting timezone identificacion')
	try:
		response = read_from_url('http://api.geonames.org/timezoneJSON?lat=%s&lng=%s&username=atareao'%(lat,lon)).decode()
		json_response = json.loads(response)		
		if json_response and 'timezoneId' in json_response.keys():
			return json_response['timezoneId'];
	except Exception as e:
		print('Error requesting timezone identification: %s'%(str(e)))
	return 'Europe/London'

def get_rawOffset(timezoneId):
	print('****** Calculating rawOffset')
	if timezoneId is not None:
		try:
			timezone = pytz.timezone(timezoneId)
			timeinzone = timezone.localize(datetime.datetime.now())
			nowdt = timeinzone.strftime('%z')
			h = s2f(nowdt)
			m = (h - int(h))/60.0
			h = int(h/100.0)+m
			return h
		except Exception as e:
			print('Error calculating rawOffset: %s'%(str(e)))
	return 0.0

'''
def get_rawOffset(lat,lon):	
	try:
		print('Requesting timezone')
		response = read_from_url('http://api.geonames.org/timezoneJSON?lat=%s&lng=%s&username=atareao'%(lat,lon)).decode()
		json_response = json.loads(response)		
		if json_response and 'timezoneId' in json_response.keys():
			timezone = pytz.timezone(json_response['timezoneId']);
			hh,mm = (str(timezone.localize(datetime.datetime.now())).split('+'))[1].split(':')
			hh = s2f(hh) + s2f(mm)/60.0#.strftime("%z");
			return hh
		return 0.0
	except:
		print('Geonames error')
		url = 'http://www.earthtools.org/timezone/%s/%s'%(lat,lon)
		response = read_from_url(url).decode()
		inicio = response.find('<offset>')+8
		fin = response.find('</offset>')
		ro = s2f(response[inicio:fin])
		inicio = response.find('<dst>')+5
		fin = response.find('</dst>')
		if response[inicio:fin]=='True':
			ro = ro + 1.0
		return ro
'''

def get_inv_direction(lat,lon):
	directions = get_inv_directions(lat,lon)
	if len(directions)>0:
		return directions[0]
	return None

def get_directions(search_string):
	directions = []
	try:
		url = URLDIR_YAHOO % (search_string,LANG)
		print('Searching url: %s'%(url))
		yahooResponse = read_from_url(url)
		if sys.version_info[0] == 3:
			jsonResponse = json.loads(yahooResponse.decode())
		else:
			jsonResponse = json.loads(yahooResponse)		
		if jsonResponse['query']['count']>1:
			for ans in jsonResponse['query']['results']['Result']:
				directions.append(fromjson2direction(ans))
		else:
			ans = jsonResponse['query']['results']['Result']
			directions.append(fromjson2direction(ans))
	except Exception as e:
		print('Error:',e)
	return directions

def get_inv_directions(lat,lon):
	directions = []
	try:
		url = URLINV_YAHOO % (lat,lon,LANG)
		print('Searching url: %s'%(url))
		yahooResponse = read_from_url(url)
		if sys.version_info[0] == 3:
			jsonResponse = json.loads(yahooResponse.decode())
		else:
			jsonResponse = json.loads(yahooResponse)
		print(jsonResponse)
		if jsonResponse['query']['count']>1:
			for ans in jsonResponse['query']['results']['Result']:
				directions.append(fromjson2direction(ans))
		else:
			ans = jsonResponse['query']['results']['Result']
			directions.append(fromjson2direction(ans))
	except Exception as e:
		print('Error:',e)
	return directions


if __name__ == "__main__":	
	print(get_timezoneId(28.63098,77.21725))
	print(get_rawOffset(get_timezoneId(28.63098,77.21725)))
	print('***********')
	print(get_timezoneId(34.2,58.3))
	print(get_rawOffset(get_timezoneId(-34.60851,-58.37349)))

	"""
	direc3 = get_inv_direction(39.4697524227712, -0.377386808395386)
	print(direc3)
	print(get_directions('valencia'))
	"""
